@@ -1,7 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2009, Kovid Goyal <kovid at kovidgoyal.net>
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 import errno
 import hashlib
@@ -163,7 +163,7 @@ class TerminalController:  # {{{
         if isinstance(cap_name, bytes):
             cap_name = cap_name.decode('utf-8')
         cap = self._escape_code(curses.tigetstr(cap_name))
-        return re.sub(r'\$<\d+>[/*]?', b'', cap)
+        return re.sub(r'\$<\d+>[/*]?', '', cap)
 
     def render(self, template):
         return re.sub(r'\$\$|\${\w+}', self._render_sub, template)
