@@ -1,15 +1,16 @@
-from __future__ import with_statement
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import os, shutil, traceback, functools, sys
 from collections import defaultdict
+from itertools import chain
 
 from calibre.customize import (CatalogPlugin, FileTypePlugin, PluginNotFound,
                               MetadataReaderPlugin, MetadataWriterPlugin,
                               InterfaceActionBase as InterfaceAction,
                               PreferencesPlugin, platform, InvalidPlugin,
-                              StoreBase as Store, ViewerPlugin, EditBookToolPlugin,
+                              StoreBase as Store, EditBookToolPlugin,
                               LibraryClosedPlugin)
 from calibre.customize.conversion import InputFormatPlugin, OutputFormatPlugin
 from calibre.customize.zipplugin import loader
@@ -20,9 +21,11 @@ from calibre.ebooks.metadata import MetaInformation
 from calibre.utils.config import (make_config_dir, Config, ConfigProxy,
                                  plugin_dir, OptionParser)
 from calibre.ebooks.metadata.sources.base import Source
-from calibre.constants import DEBUG
+from calibre.constants import DEBUG, numeric_version
+from polyglot.builtins import iteritems, itervalues, unicode_type
 
-builtin_names = frozenset([p.name for p in builtin_plugins])
+builtin_names = frozenset(p.name for p in builtin_plugins)
+BLACKLISTED_PLUGINS = frozenset({'Marvin XD', 'iOS reader applications'})
 
 
 class NameConflict(ValueError):
@@ -34,8 +37,8 @@ def _config():
     c.add_opt('plugins', default={}, help=_('Installed plugins'))
     c.add_opt('filetype_mapping', default={}, help=_('Mapping for filetype plugins'))
     c.add_opt('plugin_customization', default={}, help=_('Local plugin customization'))
-    c.add_opt('disabled_plugins', default=set([]), help=_('Disabled plugins'))
-    c.add_opt('enabled_plugins', default=set([]), help=_('Enabled plugins'))
+    c.add_opt('disabled_plugins', default=set(), help=_('Disabled plugins'))
+    c.add_opt('enabled_plugins', default=set(), help=_('Enabled plugins'))
 
     return ConfigProxy(c)
 
@@ -51,7 +54,7 @@ def find_plugin(name):
 
 def load_plugin(path_to_zip_file):  # {{{
     '''
-    Load plugin from zip file or raise InvalidPlugin error
+    Load plugin from ZIP file or raise InvalidPlugin error
 
     :return: A :class:`Plugin` instance.
     '''
@@ -99,9 +102,9 @@ def restore_plugin_state_to_default(plugin_or_name):
     config['enabled_plugins'] = ep
 
 
-default_disabled_plugins = set([
+default_disabled_plugins = {
     'Overdrive', 'Douban Books', 'OZON.ru', 'Edelweiss', 'Google Images', 'Big Book Search',
-])
+}
 
 
 def is_disabled(plugin):
@@ -122,11 +125,7 @@ _on_postadd          = []
 
 
 def reread_filetype_plugins():
-    global _on_import
-    global _on_postimport
-    global _on_preprocess
-    global _on_postprocess
-    global _on_postadd
+    global _on_import, _on_postimport, _on_preprocess, _on_postprocess, _on_postadd
     _on_import           = defaultdict(list)
     _on_postimport       = defaultdict(list)
     _on_preprocess       = defaultdict(list)
@@ -147,16 +146,21 @@ def reread_filetype_plugins():
                     _on_postprocess[ft].append(plugin)
 
 
+def plugins_for_ft(ft, occasion):
+    op = {
+        'import':_on_import, 'preprocess':_on_preprocess, 'postprocess':_on_postprocess, 'postimport':_on_postimport,
+    }[occasion]
+    for p in chain(op.get(ft, ()), op.get('*', ())):
+        if not is_disabled(p):
+            yield p
+
+
 def _run_filetype_plugins(path_to_file, ft=None, occasion='preprocess'):
-    occasion_plugins = {'import':_on_import, 'preprocess':_on_preprocess,
-                'postprocess':_on_postprocess}[occasion]
     customization = config['plugin_customization']
     if ft is None:
         ft = os.path.splitext(path_to_file)[-1].lower().replace('.', '')
     nfp = path_to_file
-    for plugin in occasion_plugins.get(ft, []):
-        if is_disabled(plugin):
-            continue
+    for plugin in plugins_for_ft(ft, occasion):
         plugin.site_customization = customization.get(plugin.name, '')
         oo, oe = sys.stdout, sys.stderr  # Some file type plugins out there override the output streams with buggy implementations
         with plugin:
@@ -167,36 +171,31 @@ def _run_filetype_plugins(path_to_file, ft=None, occasion='preprocess'):
             try:
                 nfp = plugin.run(nfp) or nfp
             except:
-                print >>oe, 'Running file type plugin %s failed with traceback:'%plugin.name
+                print('Running file type plugin %s failed with traceback:'%plugin.name, file=oe)
                 traceback.print_exc(file=oe)
         sys.stdout, sys.stderr = oo, oe
-    x = lambda j : os.path.normpath(os.path.normcase(j))
+    x = lambda j: os.path.normpath(os.path.normcase(j))
     if occasion == 'postprocess' and x(nfp) != x(path_to_file):
         shutil.copyfile(nfp, path_to_file)
         nfp = path_to_file
     return nfp
 
 
-run_plugins_on_import      = functools.partial(_run_filetype_plugins,
-                                               occasion='import')
-run_plugins_on_preprocess  = functools.partial(_run_filetype_plugins,
-                                               occasion='preprocess')
-run_plugins_on_postprocess = functools.partial(_run_filetype_plugins,
-                                               occasion='postprocess')
+run_plugins_on_import      = functools.partial(_run_filetype_plugins, occasion='import')
+run_plugins_on_preprocess  = functools.partial(_run_filetype_plugins, occasion='preprocess')
+run_plugins_on_postprocess = functools.partial(_run_filetype_plugins, occasion='postprocess')
 
 
 def run_plugins_on_postimport(db, book_id, fmt):
     customization = config['plugin_customization']
     fmt = fmt.lower()
-    for plugin in _on_postimport.get(fmt, []):
-        if is_disabled(plugin):
-            continue
+    for plugin in plugins_for_ft(fmt, 'postimport'):
         plugin.site_customization = customization.get(plugin.name, '')
         with plugin:
             try:
                 plugin.postimport(book_id, fmt, db)
             except:
-                print ('Running file type plugin %s failed with traceback:'%
+                print('Running file type plugin %s failed with traceback:'%
                        plugin.name)
                 traceback.print_exc()
 
@@ -211,7 +210,7 @@ def run_plugins_on_postadd(db, book_id, fmt_map):
             try:
                 plugin.postadd(book_id, fmt_map, db)
             except Exception:
-                print ('Running file type plugin %s failed with traceback:'%
+                print('Running file type plugin %s failed with traceback:'%
                        plugin.name)
                 traceback.print_exc()
 
@@ -308,14 +307,14 @@ def available_store_plugins():
 
 
 def stores():
-    stores = set([])
+    stores = set()
     for plugin in store_plugins():
         stores.add(plugin.name)
     return stores
 
 
 def available_stores():
-    stores = set([])
+    stores = set()
     for plugin in available_store_plugins():
         stores.add(plugin.name)
     return stores
@@ -342,9 +341,19 @@ def reread_metadata_plugins():
             for ft in plugin.file_types:
                 _metadata_writers[ft].append(plugin)
 
+    # Ensure custom metadata plugins are used in preference to builtin
+    # ones for a given filetype
+    def key(plugin):
+        return (1 if plugin.plugin_path is None else 0), plugin.name
+
+    for group in (_metadata_readers, _metadata_writers):
+        for plugins in itervalues(group):
+            if len(plugins) > 1:
+                plugins.sort(key=key)
+
 
 def metadata_readers():
-    ans = set([])
+    ans = set()
     for plugins in _metadata_readers.values():
         for plugin in plugins:
             ans.add(plugin)
@@ -352,7 +361,7 @@ def metadata_readers():
 
 
 def metadata_writers():
-    ans = set([])
+    ans = set()
     for plugins in _metadata_writers.values():
         for plugin in plugins:
             ans.add(plugin)
@@ -548,7 +557,7 @@ def plugin_for_output_format(fmt):
 
 
 def available_output_formats():
-    formats = set([])
+    formats = set()
     for plugin in output_format_plugins():
         if not is_disabled(plugin):
             formats.add(plugin.file_type)
@@ -566,7 +575,7 @@ def catalog_plugins():
 
 
 def available_catalog_formats():
-    formats = set([])
+    formats = set()
     for plugin in catalog_plugins():
         if not is_disabled(plugin):
             for format in plugin.file_types:
@@ -618,15 +627,21 @@ def all_metadata_plugins():
     for plugin in _initialized_plugins:
         if isinstance(plugin, Source):
             yield plugin
-# }}}
-
-# Viewer plugins {{{
 
 
-def all_viewer_plugins():
-    for plugin in _initialized_plugins:
-        if isinstance(plugin, ViewerPlugin):
-            yield plugin
+def patch_metadata_plugins(possibly_updated_plugins):
+    patches = {}
+    for i, plugin in enumerate(_initialized_plugins):
+        if isinstance(plugin, Source) and plugin.name in builtin_names:
+            pup = possibly_updated_plugins.get(plugin.name)
+            if pup is not None:
+                if pup.version > plugin.version and pup.minimum_calibre_version <= numeric_version:
+                    patches[i] = pup(None)
+                    # Metadata source plugins dont use initialize() but that
+                    # might change in the future, so be safe.
+                    patches[i].initialize()
+    for i, pup in iteritems(patches):
+        _initialized_plugins[i] = pup
 # }}}
 
 # Editor plugins {{{
@@ -650,14 +665,14 @@ def initialize_plugin(plugin, path_to_zip_file):
         p.initialize()
         return p
     except Exception:
-        print 'Failed to initialize plugin:', plugin.name, plugin.version
+        print('Failed to initialize plugin:', plugin.name, plugin.version)
         tb = traceback.format_exc()
         raise InvalidPlugin((_('Initialization of plugin %s failed with traceback:')
                             %tb) + '\n'+tb)
 
 
 def has_external_plugins():
-    'True if there are updateable (zip file based) plugins'
+    'True if there are updateable (ZIP file based) plugins'
     return bool(config['plugins'])
 
 
@@ -668,7 +683,9 @@ def initialize_plugins(perf=False):
             builtin_names]
     for p in conflicts:
         remove_plugin(p)
-    external_plugins = config['plugins']
+    external_plugins = config['plugins'].copy()
+    for name in BLACKLISTED_PLUGINS:
+        external_plugins.pop(name, None)
     ostdout, ostderr = sys.stdout, sys.stderr
     if perf:
         from collections import defaultdict
@@ -693,16 +710,16 @@ def initialize_plugins(perf=False):
                 times[plugin.name] = time.time() - st
             _initialized_plugins.append(plugin)
         except:
-            print 'Failed to initialize plugin:', repr(zfp)
+            print('Failed to initialize plugin:', repr(zfp))
             if DEBUG:
                 traceback.print_exc()
     # Prevent a custom plugin from overriding stdout/stderr as this breaks
     # ipython
     sys.stdout, sys.stderr = ostdout, ostderr
     if perf:
-        for x in sorted(times, key=lambda x:times[x]):
-            print ('%50s: %.3f'%(x, times[x]))
-    _initialized_plugins.sort(cmp=lambda x,y:cmp(x.priority, y.priority), reverse=True)
+        for x in sorted(times, key=lambda x: times[x]):
+            print('%50s: %.3f'%(x, times[x]))
+    _initialized_plugins.sort(key=lambda x: x.priority, reverse=True)
     reread_filetype_plugins()
     reread_metadata_plugins()
 
@@ -723,18 +740,18 @@ def build_plugin(path):
     from calibre import prints
     from calibre.ptempfile import PersistentTemporaryFile
     from calibre.utils.zipfile import ZipFile, ZIP_STORED
-    path = type(u'')(path)
+    path = unicode_type(path)
     names = frozenset(os.listdir(path))
-    if u'__init__.py' not in names:
+    if '__init__.py' not in names:
         prints(path, ' is not a valid plugin')
         raise SystemExit(1)
     t = PersistentTemporaryFile(u'.zip')
-    with ZipFile(t, u'w', ZIP_STORED) as zf:
+    with ZipFile(t, 'w', ZIP_STORED) as zf:
         zf.add_dir(path, simple_filter=lambda x:x in {'.git', '.bzr', '.svn', '.hg'})
     t.close()
     plugin = add_plugin(t.name)
     os.remove(t.name)
-    prints(u'Plugin updated:', plugin.name, plugin.version)
+    prints('Plugin updated:', plugin.name, plugin.version)
 
 
 def option_parser():
@@ -744,7 +761,7 @@ def option_parser():
     Customize calibre by loading external plugins.
     '''))
     parser.add_option('-a', '--add-plugin', default=None,
-                      help=_('Add a plugin by specifying the path to the zip file containing it.'))
+                      help=_('Add a plugin by specifying the path to the ZIP file containing it.'))
     parser.add_option('-b', '--build-plugin', default=None,
             help=_('For plugin developers: Path to the directory where you are'
                 ' developing the plugin. This command will automatically zip '
@@ -770,19 +787,19 @@ def main(args=sys.argv):
     opts, args = parser.parse_args(args)
     if opts.add_plugin is not None:
         plugin = add_plugin(opts.add_plugin)
-        print 'Plugin added:', plugin.name, plugin.version
+        print('Plugin added:', plugin.name, plugin.version)
     if opts.build_plugin is not None:
         build_plugin(opts.build_plugin)
     if opts.remove_plugin is not None:
         if remove_plugin(opts.remove_plugin):
-            print 'Plugin removed'
+            print('Plugin removed')
         else:
-            print 'No custom plugin named', opts.remove_plugin
+            print('No custom plugin named', opts.remove_plugin)
     if opts.customize_plugin is not None:
         name, custom = opts.customize_plugin.split(',')
         plugin = find_plugin(name.strip())
         if plugin is None:
-            print 'No plugin with the name %s exists'%name
+            print('No plugin with the name %s exists'%name)
             return 1
         customize_plugin(plugin, custom)
     if opts.enable_plugin is not None:
@@ -794,21 +811,21 @@ def main(args=sys.argv):
         for plugin in initialized_plugins():
             type_len, name_len = max(type_len, len(plugin.type)), max(name_len, len(plugin.name))
         fmt = '%-{}s%-{}s%-15s%-15s%s'.format(type_len+1, name_len+1)
-        print fmt%tuple(('Type|Name|Version|Disabled|Site Customization'.split('|')))
-        print
+        print(fmt%tuple(('Type|Name|Version|Disabled|Site Customization'.split('|'))))
+        print()
         for plugin in initialized_plugins():
-            print fmt%(
+            print(fmt%(
                                 plugin.type, plugin.name,
                                 plugin.version, is_disabled(plugin),
                                 plugin_customization(plugin)
-                                )
-            print '\t', plugin.description
+                                ))
+            print('\t', plugin.description)
             if plugin.is_customizable():
                 try:
-                    print '\t', plugin.customization_help()
+                    print('\t', plugin.customization_help())
                 except NotImplementedError:
                     pass
-            print
+            print()
 
     return 0
 
