@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -19,6 +18,7 @@ from calibre.utils.date import parse_date, UNDEFINED_DATE, now, dt_as_local
 from calibre.utils.icu import primary_contains, sort_key
 from calibre.utils.localization import lang_map, canonicalize_lang
 from calibre.utils.search_query_parser import SearchQueryParser, ParseException
+from polyglot.builtins import iteritems, unicode_type, string_or_bytes
 
 CONTAINS_MATCH = 0
 EQUALS_MATCH   = 1
@@ -26,7 +26,8 @@ REGEXP_MATCH   = 2
 
 # Utils {{{
 
-def _matchkind(query):
+
+def _matchkind(query, case_sensitive=False):
     matchkind = CONTAINS_MATCH
     if (len(query) > 1):
         if query.startswith('\\'):
@@ -38,12 +39,13 @@ def _matchkind(query):
             matchkind = REGEXP_MATCH
             query = query[1:]
 
-    if matchkind != REGEXP_MATCH:
+    if not case_sensitive and matchkind != REGEXP_MATCH:
         # leave case in regexps because it can be significant e.g. \S \W \D
         query = icu_lower(query)
     return matchkind, query
 
-def _match(query, value, matchkind, use_primary_find_in_search=True):
+
+def _match(query, value, matchkind, use_primary_find_in_search=True, case_sensitive=False):
     if query.startswith('..'):
         query = query[1:]
         sq = query[1:]
@@ -52,7 +54,8 @@ def _match(query, value, matchkind, use_primary_find_in_search=True):
         internal_match_ok = False
     for t in value:
         try:  # ignore regexp exceptions, required because search-ahead tries before typing is finished
-            t = icu_lower(t)
+            if not case_sensitive:
+                t = icu_lower(t)
             if (matchkind == EQUALS_MATCH):
                 if internal_match_ok:
                     if query == t:
@@ -69,19 +72,20 @@ def _match(query, value, matchkind, use_primary_find_in_search=True):
                 elif query == t:
                     return True
             elif matchkind == REGEXP_MATCH:
-                if re.search(query, t, re.I|re.UNICODE):
+                if re.search(query, t, re.UNICODE if case_sensitive else re.I|re.UNICODE):
                     return True
             elif matchkind == CONTAINS_MATCH:
-                if use_primary_find_in_search:
+                if not case_sensitive and use_primary_find_in_search:
                     if primary_contains(query, t):
                         return True
                 elif query in t:
-                        return True
+                    return True
         except re.error:
             pass
     return False
 # }}}
 
+
 class DateSearch(object):  # {{{
 
     def __init__(self):
@@ -144,7 +148,9 @@ class DateSearch(object):  # {{{
 
         if query == 'false':
             for v, book_ids in field_iter():
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
+                    if isinstance(v, bytes):
+                        v = v.decode(preferred_encoding, 'replace')
                     v = parse_date(v)
                 if v is None or v <= UNDEFINED_DATE:
                     matches |= book_ids
@@ -152,13 +158,15 @@ class DateSearch(object):  # {{{
 
         if query == 'true':
             for v, book_ids in field_iter():
-                if isinstance(v, (str, unicode)):
+                if isinstance(v, (bytes, unicode_type)):
+                    if isinstance(v, bytes):
+                        v = v.decode(preferred_encoding, 'replace')
                     v = parse_date(v)
                 if v is not None and v > UNDEFINED_DATE:
                     matches |= book_ids
             return matches
 
-        for k, relop in self.operators.iteritems():
+        for k, relop in iteritems(self.operators):
             if query.startswith(k):
                 query = query[len(k):]
                 break
@@ -194,7 +202,7 @@ class DateSearch(object):  # {{{
                     field_count = query.count('/') + 1
 
         for v, book_ids in field_iter():
-            if isinstance(v, (str, unicode)):
+            if isinstance(v, string_or_bytes):
                 v = parse_date(v)
             if v is not None and relop(dt_as_local(v), qd, field_count):
                 matches |= book_ids
@@ -202,6 +210,7 @@ class DateSearch(object):  # {{{
         return matches
 # }}}
 
+
 class NumericSearch(object):  # {{{
 
     def __init__(self):
@@ -244,19 +253,18 @@ class NumericSearch(object):  # {{{
             else:
                 relop = lambda x,y: x is not None
         else:
-            for k, relop in self.operators.iteritems():
+            for k, relop in iteritems(self.operators):
                 if query.startswith(k):
                     query = query[len(k):]
                     break
             else:
                 relop = self.operators['=']
 
-            cast = int
             if dt == 'rating':
                 cast = lambda x: 0 if x is None else int(x)
                 adjust = lambda x: x // 2
-            elif dt in ('float', 'composite'):
-                cast = float
+            else:
+                cast = float if dt in ('float', 'composite', 'half-rating') else int
 
             mult = 1.0
             if len(query) > 1:
@@ -269,9 +277,12 @@ class NumericSearch(object):  # {{{
 
             try:
                 q = cast(query) * mult
-            except:
+            except Exception:
                 raise ParseException(
                         _('Non-numeric value in query: {0}').format(query))
+            if dt == 'half-rating':
+                q = int(round(q * 2))
+                cast = int
 
         qfalse = query == 'false'
         for val, book_ids in field_iter():
@@ -281,7 +292,7 @@ class NumericSearch(object):  # {{{
                 continue
             try:
                 v = cast(val)
-            except:
+            except Exception:
                 v = None
             if v:
                 v = adjust(v)
@@ -291,6 +302,7 @@ class NumericSearch(object):  # {{{
 
 # }}}
 
+
 class BooleanSearch(object):  # {{{
 
     def __init__(self):
@@ -332,6 +344,7 @@ class BooleanSearch(object):  # {{{
 
 # }}}
 
+
 class KeyPairSearch(object):  # {{{
 
     def __call__(self, query, field_iter, candidates, use_primary_find):
@@ -358,7 +371,7 @@ class KeyPairSearch(object):  # {{{
             return found if valq == 'true' else candidates - found
 
         for m, book_ids in field_iter():
-            for key, val in m.iteritems():
+            for key, val in iteritems(m):
                 if (keyq and not _match(keyq, (key,), keyq_mkind,
                                         use_primary_find_in_search=use_primary_find)):
                     continue
@@ -372,6 +385,7 @@ class KeyPairSearch(object):  # {{{
 
 # }}}
 
+
 class SavedSearchQueries(object):  # {{{
     queries = {}
     opt_name = ''
@@ -397,7 +411,7 @@ class SavedSearchQueries(object):  # {{{
         return self._db()
 
     def force_unicode(self, x):
-        if not isinstance(x, unicode):
+        if not isinstance(x, unicode_type):
             x = x.decode(preferred_encoding, 'replace')
         return x
 
@@ -430,9 +444,10 @@ class SavedSearchQueries(object):  # {{{
             db._set_pref(self.opt_name, smap)
 
     def names(self):
-        return sorted(self.queries.iterkeys(), key=sort_key)
+        return sorted(self.queries, key=sort_key)
 # }}}
 
+
 class Parser(SearchQueryParser):  # {{{
 
     def __init__(self, dbcache, all_book_ids, gst, date_search, num_search,
@@ -487,6 +502,15 @@ class Parser(SearchQueryParser):  # {{{
         if location not in self.all_search_locations:
             return matches
 
+        if location == 'vl':
+            vl = self.dbcache._pref('virtual_libraries', {}).get(query) if query else None
+            if not vl:
+                raise ParseException(_('No such virtual library: {}').format(query))
+            try:
+                return candidates & self.dbcache.books_in_virtual_library(query)
+            except RuntimeError:
+                raise ParseException(_('Virtual library search is recursive: {}').format(query))
+
         if (len(location) > 2 and location.startswith('@') and
                     location[1:] in self.grouped_search_terms):
             location = location[1:]
@@ -562,12 +586,15 @@ class Parser(SearchQueryParser):  # {{{
                      fm['display'].get('composite_sort', '') == 'number')):
                 if location == 'id':
                     is_many = False
+
                     def fi(default_value=None):
                         for qid in candidates:
                             yield qid, {qid}
                 else:
                     field = self.dbcache.fields[location]
                     fi, is_many = partial(self.field_iter, location, candidates), field.is_many
+                if dt == 'rating' and fm['display'].get('allow_half_stars'):
+                    dt = 'half-rating'
                 return self.num_search(
                     icu_lower(query), fi, location, dt, candidates, is_many=is_many)
 
@@ -598,12 +625,13 @@ class Parser(SearchQueryParser):  # {{{
             return self.get_user_category_matches(location[1:], icu_lower(query), candidates)
 
         # Everything else (and 'all' matches)
-        matchkind, query = _matchkind(query)
+        case_sensitive = prefs['case_sensitive']
+        matchkind, query = _matchkind(query, case_sensitive=case_sensitive)
         all_locs = set()
         text_fields = set()
         field_metadata = {}
 
-        for x, fm in self.field_metadata.iteritems():
+        for x, fm in self.field_metadata.iter_items():
             if x.startswith('@'):
                 continue
             if fm['search_terms'] and x not in {'series_sort', 'id'}:
@@ -641,15 +669,15 @@ class Parser(SearchQueryParser):  # {{{
                 q = canonicalize_lang(query)
                 if q is None:
                     lm = lang_map()
-                    rm = {v.lower():k for k,v in lm.iteritems()}
+                    rm = {v.lower():k for k,v in iteritems(lm)}
                     q = rm.get(query, query)
 
-            if matchkind == CONTAINS_MATCH and q in {'true', 'false'}:
+            if matchkind == CONTAINS_MATCH and q.lower() in {'true', 'false'}:
                 found = set()
                 for val, book_ids in self.field_iter(location, current_candidates):
                     if val and (not hasattr(val, 'strip') or val.strip()):
                         found |= book_ids
-                matches |= (found if q == 'true' else (current_candidates-found))
+                matches |= (found if q.lower() == 'true' else (current_candidates-found))
                 continue
 
             dt = field_metadata.get(location, {}).get('datatype', None)
@@ -677,16 +705,16 @@ class Parser(SearchQueryParser):  # {{{
             if location in text_fields:
                 for val, book_ids in self.field_iter(location, current_candidates):
                     if val is not None:
-                        if isinstance(val, basestring):
+                        if isinstance(val, string_or_bytes):
                             val = (val,)
-                        if _match(q, val, matchkind, use_primary_find_in_search=upf):
+                        if _match(q, val, matchkind, use_primary_find_in_search=upf, case_sensitive=case_sensitive):
                             matches |= book_ids
 
             if location == 'series_sort':
                 book_lang_map = self.dbcache.fields['languages'].book_value_map
                 for val, book_ids in self.dbcache.fields['series'].iter_searchable_values_for_sort(current_candidates, book_lang_map):
                     if val is not None:
-                        if _match(q, (val,), matchkind, use_primary_find_in_search=upf):
+                        if _match(q, (val,), matchkind, use_primary_find_in_search=upf, case_sensitive=case_sensitive):
                             matches |= book_ids
 
         return matches
@@ -716,6 +744,7 @@ class Parser(SearchQueryParser):  # {{{
         return matches
 # }}}
 
+
 class LRUCache(object):  # {{{
 
     'A simple Least-Recently-Used cache'
@@ -769,9 +798,10 @@ class LRUCache(object):  # {{{
         return self.get(key)
 
     def __iter__(self):
-        return self.item_map.iteritems()
+        return iteritems(self.item_map)
 # }}}
 
+
 class Search(object):
 
     MAX_CACHE_UPDATE = 50
@@ -898,4 +928,3 @@ class Search(object):
             self.cache.add(query, result)
 
         return result
-
