@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -11,9 +10,11 @@ from collections import namedtuple
 from functools import partial
 from io import BytesIO
 
-from calibre.ebooks.metadata import author_to_author_sort
+from calibre.ebooks.metadata import author_to_author_sort, title_sort
 from calibre.utils.date import UNDEFINED_DATE
 from calibre.db.tests.base import BaseTest, IMG
+from polyglot.builtins import iteritems, itervalues, unicode_type
+
 
 class WritingTest(BaseTest):
 
@@ -151,8 +152,8 @@ class WritingTest(BaseTest):
         del cache2
         self.assertEqual(cache.set_field('publisher', {1:'one', 2:'two',
                                                        3:'three'}), {1, 2, 3})
-        self.assertEqual(cache.set_field('publisher', {1:''}), set([1]))
-        self.assertEqual(cache.set_field('publisher', {1:'two'}), set([1]))
+        self.assertEqual(cache.set_field('publisher', {1:''}), {1})
+        self.assertEqual(cache.set_field('publisher', {1:'two'}), {1})
         self.assertEqual(tuple(map(f.for_book, (1,2,3))), ('two', 'two', 'three'))
         self.assertEqual(cache.set_field('publisher', {1:'Two'}), {1, 2})
         cache2 = self.init_cache(cl)
@@ -162,10 +163,10 @@ class WritingTest(BaseTest):
         # Enum
         self.assertFalse(cache.set_field('#enum', {1:'Not allowed'}))
         self.assertEqual(cache.set_field('#enum', {1:'One', 2:'One', 3:'Three'}), {1, 3})
-        self.assertEqual(cache.set_field('#enum', {1:None}), set([1]))
+        self.assertEqual(cache.set_field('#enum', {1:None}), {1})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
-            for i, val in {1:None, 2:'One', 3:'Three'}.iteritems():
+            for i, val in iteritems({1:None, 2:'One', 3:'Three'}):
                 self.assertEqual(c.field_for('#enum', i), val)
         del cache2
 
@@ -175,29 +176,29 @@ class WritingTest(BaseTest):
         self.assertEqual(cache.set_field('#rating', {1:None, 2:4, 3:8}), {1, 2, 3})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
-            for i, val in {1:None, 2:4, 3:2}.iteritems():
+            for i, val in iteritems({1:None, 2:4, 3:2}):
                 self.assertEqual(c.field_for('rating', i), val)
-            for i, val in {1:None, 2:4, 3:8}.iteritems():
+            for i, val in iteritems({1:None, 2:4, 3:8}):
                 self.assertEqual(c.field_for('#rating', i), val)
         del cache2
 
         # Series
         self.assertFalse(cache.set_field('series',
                 {1:'a series one', 2:'a series one'}, allow_case_change=False))
-        self.assertEqual(cache.set_field('series', {3:'Series [3]'}), set([3]))
+        self.assertEqual(cache.set_field('series', {3:'Series [3]'}), {3})
         self.assertEqual(cache.set_field('#series', {1:'Series', 3:'Series'}),
                                          {1, 3})
-        self.assertEqual(cache.set_field('#series', {2:'Series [0]'}), set([2]))
+        self.assertEqual(cache.set_field('#series', {2:'Series [0]'}), {2})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
-            for i, val in {1:'A Series One', 2:'A Series One', 3:'Series'}.iteritems():
+            for i, val in iteritems({1:'A Series One', 2:'A Series One', 3:'Series'}):
                 self.assertEqual(c.field_for('series', i), val)
             cs_indices = {1:c.field_for('#series_index', 1), 3:c.field_for('#series_index', 3)}
             for i in (1, 2, 3):
                 self.assertEqual(c.field_for('#series', i), 'Series')
-            for i, val in {1:2, 2:1, 3:3}.iteritems():
+            for i, val in iteritems({1:2, 2:1, 3:3}):
                 self.assertEqual(c.field_for('series_index', i), val)
-            for i, val in {1:cs_indices[1], 2:0, 3:cs_indices[3]}.iteritems():
+            for i, val in iteritems({1:cs_indices[1], 2:0, 3:cs_indices[3]}):
                 self.assertEqual(c.field_for('#series_index', i), val)
         del cache2
 
@@ -218,7 +219,7 @@ class WritingTest(BaseTest):
             ae(sf(name, {1:'tag one, News'}), {1, 2})
             ae(sf(name, {3:('tag two', 'sep,sep2')}), {2, 3})
             ae(len(f.table.id_map), 4)
-            ae(sf(name, {1:None}), set([1]))
+            ae(sf(name, {1:None}), {1})
             cache2 = self.init_cache(cl)
             for c in (cache, cache2):
                 ae(c.field_for(name, 3), ('tag two', 'sep;sep2'))
@@ -235,17 +236,17 @@ class WritingTest(BaseTest):
         for name in ('authors', '#authors'):
             f = cache.fields[name]
             ae(len(f.table.id_map), 3)
-            af(cache.set_field(name, {3:None if name == 'authors' else 'Unknown'}))
-            ae(cache.set_field(name, {3:'Kovid Goyal & Divok Layog'}), set([3]))
+            af(cache.set_field(name, {3:'Unknown'}))
+            ae(cache.set_field(name, {3:'Kovid Goyal & Divok Layog'}), {3})
             ae(cache.set_field(name, {1:'', 2:'An, Author'}), {1,2})
             cache2 = self.init_cache(cl)
             for c in (cache, cache2):
                 ae(len(c.fields[name].table.id_map), 4 if name =='authors' else 3)
                 ae(c.field_for(name, 3), ('Kovid Goyal', 'Divok Layog'))
                 ae(c.field_for(name, 2), ('An, Author',))
-                ae(c.field_for(name, 1), ('Unknown',) if name=='authors' else ())
+                ae(c.field_for(name, 1), (_('Unknown'),) if name=='authors' else ())
                 if name == 'authors':
-                    ae(c.field_for('author_sort', 1), author_to_author_sort('Unknown'))
+                    ae(c.field_for('author_sort', 1), author_to_author_sort(_('Unknown')))
                     ae(c.field_for('author_sort', 2), author_to_author_sort('An, Author'))
                     ae(c.field_for('author_sort', 3), author_to_author_sort('Kovid Goyal') + ' & ' + author_to_author_sort('Divok Layog'))
             del cache2
@@ -256,18 +257,18 @@ class WritingTest(BaseTest):
         # Languages
         f = cache.fields['languages']
         ae(f.table.id_map, {1: 'eng', 2: 'deu'})
-        ae(sf('languages', {1:''}), set([1]))
+        ae(sf('languages', {1:''}), {1})
         ae(cache.field_for('languages', 1), ())
-        ae(sf('languages', {2:('und',)}), set([2]))
+        ae(sf('languages', {2:('und',)}), {2})
         af(f.table.id_map)
         ae(sf('languages', {1:'eng,fra,deu', 2:'es,Dutch', 3:'English'}), {1, 2, 3})
         ae(cache.field_for('languages', 1), ('eng', 'fra', 'deu'))
         ae(cache.field_for('languages', 2), ('spa', 'nld'))
         ae(cache.field_for('languages', 3), ('eng',))
-        ae(sf('languages', {3:None}), set([3]))
+        ae(sf('languages', {3:None}), {3})
         ae(cache.field_for('languages', 3), ())
-        ae(sf('languages', {1:'deu,fra,eng'}), set([1]), 'Changing order failed')
-        ae(sf('languages', {2:'deu,eng,eng'}), set([2]))
+        ae(sf('languages', {1:'deu,fra,eng'}), {1}, 'Changing order failed')
+        ae(sf('languages', {2:'deu,eng,eng'}), {2})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
             ae(cache.field_for('languages', 1), ('deu', 'fra', 'eng'))
@@ -276,9 +277,9 @@ class WritingTest(BaseTest):
 
         # Identifiers
         f = cache.fields['identifiers']
-        ae(sf('identifiers', {3: 'one:1,two:2'}), set([3]))
-        ae(sf('identifiers', {2:None}), set([2]))
-        ae(sf('identifiers', {1: {'test':'1', 'two':'2'}}), set([1]))
+        ae(sf('identifiers', {3: 'one:1,two:2'}), {3})
+        ae(sf('identifiers', {2:None}), {2})
+        ae(sf('identifiers', {1: {'test':'1', 'two':'2'}}), {1})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
             ae(c.field_for('identifiers', 3), {'one':'1', 'two':'2'})
@@ -290,8 +291,8 @@ class WritingTest(BaseTest):
         ae(sf('title', {1:'The Moose', 2:'Cat'}), {1, 2})
         cache2 = self.init_cache(cl)
         for c in (cache, cache2):
-            ae(c.field_for('sort', 1), 'Moose, The')
-            ae(c.field_for('sort', 2), 'Cat')
+            ae(c.field_for('sort', 1), title_sort('The Moose'))
+            ae(c.field_for('sort', 2), title_sort('Cat'))
 
         # Test setting with the same value repeated
         ae(sf('tags', {3: ('a', 'b', 'a')}), {3})
@@ -322,7 +323,7 @@ class WritingTest(BaseTest):
         onowf = c.nowf
         c.nowf = lambda: utime
         try:
-            ae(sf('title', {3:'xxx'}), set([3]))
+            ae(sf('title', {3:'xxx'}), {3})
             self.assertTrue(3 in cache.dirtied_cache)
             ae(cache.field_for('last_modified', 3), utime)
             cache.dump_metadata()
@@ -373,7 +374,7 @@ class WritingTest(BaseTest):
 
         # Test removing a cover
         ae(cache.field_for('cover', 1), 1)
-        ae(cache.set_cover({1:None}), set([1]))
+        ae(cache.set_cover({1:None}), {1})
         ae(cache.field_for('cover', 1), 0)
         img = IMG
 
@@ -443,7 +444,7 @@ class WritingTest(BaseTest):
         all_ids = cache.all_book_ids()
         self.assertFalse(cache.has_conversion_options(all_ids))
         self.assertIsNone(cache.conversion_options(1))
-        op1, op2 = {'xx':'yy'}, {'yy':'zz'}
+        op1, op2 = b"{'xx':'yy'}", b"{'yy':'zz'}"
         cache.set_conversion_options({1:op1, 2:op2})
         self.assertTrue(cache.has_conversion_options(all_ids))
         self.assertEqual(cache.conversion_options(1), op1)
@@ -460,13 +461,13 @@ class WritingTest(BaseTest):
         tmap = cache.get_id_map('tags')
         self.assertEqual(cache.remove_items('tags', tmap), {1, 2})
         tmap = cache.get_id_map('#tags')
-        t = {v:k for k, v in tmap.iteritems()}['My Tag Two']
+        t = {v:k for k, v in iteritems(tmap)}['My Tag Two']
         self.assertEqual(cache.remove_items('#tags', (t,)), {1, 2})
 
         smap = cache.get_id_map('series')
         self.assertEqual(cache.remove_items('series', smap), {1, 2})
         smap = cache.get_id_map('#series')
-        s = {v:k for k, v in smap.iteritems()}['My Series Two']
+        s = {v:k for k, v in iteritems(smap)}['My Series Two']
         self.assertEqual(cache.remove_items('#series', (s,)), {1})
 
         for c in (cache, self.init_cache()):
@@ -506,7 +507,7 @@ class WritingTest(BaseTest):
         for c in (cache, c2):
             self.assertEqual(c.field_for('tags', 1), ())
             self.assertEqual(c.field_for('tags', 2), ('b', 'a'))
-            self.assertNotIn('c', set(c.get_id_map('tags').itervalues()))
+            self.assertNotIn('c', set(itervalues(c.get_id_map('tags'))))
             self.assertEqual(c.field_for('series', 1), None)
             self.assertEqual(c.field_for('series', 2), 'a')
             self.assertEqual(c.field_for('series_index', 1), 1.0)
@@ -519,9 +520,9 @@ class WritingTest(BaseTest):
         cl = self.cloned_library
         cache = self.init_cache(cl)
         # Check that renaming authors updates author sort and path
-        a = {v:k for k, v in cache.get_id_map('authors').iteritems()}['Unknown']
+        a = {v:k for k, v in iteritems(cache.get_id_map('authors'))}['Unknown']
         self.assertEqual(cache.rename_items('authors', {a:'New Author'})[0], {3})
-        a = {v:k for k, v in cache.get_id_map('authors').iteritems()}['Author One']
+        a = {v:k for k, v in iteritems(cache.get_id_map('authors'))}['Author One']
         self.assertEqual(cache.rename_items('authors', {a:'Author Two'})[0], {1, 2})
         for c in (cache, self.init_cache(cl)):
             self.assertEqual(c.all_field_names('authors'), {'New Author', 'Author Two'})
@@ -530,7 +531,7 @@ class WritingTest(BaseTest):
             self.assertEqual(c.field_for('authors', 1), ('Author Two',))
             self.assertEqual(c.field_for('author_sort', 1), 'Two, Author')
 
-        t = {v:k for k, v in cache.get_id_map('tags').iteritems()}['Tag One']
+        t = {v:k for k, v in iteritems(cache.get_id_map('tags'))}['Tag One']
         # Test case change
         self.assertEqual(cache.rename_items('tags', {t:'tag one'}), ({1, 2}, {t:t}))
         for c in (cache, self.init_cache(cl)):
@@ -550,14 +551,14 @@ class WritingTest(BaseTest):
             self.assertEqual(set(c.field_for('tags', 1)), {'Tag Two', 'News'})
             self.assertEqual(set(c.field_for('tags', 2)), {'Tag Two'})
         # Test on a custom column
-        t = {v:k for k, v in cache.get_id_map('#tags').iteritems()}['My Tag One']
+        t = {v:k for k, v in iteritems(cache.get_id_map('#tags'))}['My Tag One']
         self.assertEqual(cache.rename_items('#tags', {t:'My Tag Two'})[0], {2})
         for c in (cache, self.init_cache(cl)):
             self.assertEqual(c.all_field_names('#tags'), {'My Tag Two'})
             self.assertEqual(set(c.field_for('#tags', 2)), {'My Tag Two'})
 
         # Test a Many-one field
-        s = {v:k for k, v in cache.get_id_map('series').iteritems()}['A Series One']
+        s = {v:k for k, v in iteritems(cache.get_id_map('series'))}['A Series One']
         # Test case change
         self.assertEqual(cache.rename_items('series', {s:'a series one'}), ({1, 2}, {s:s}))
         for c in (cache, self.init_cache(cl)):
@@ -573,7 +574,7 @@ class WritingTest(BaseTest):
             self.assertEqual(c.field_for('series', 2), 'series')
             self.assertEqual(c.field_for('series_index', 1), 2.0)
 
-        s = {v:k for k, v in cache.get_id_map('#series').iteritems()}['My Series One']
+        s = {v:k for k, v in iteritems(cache.get_id_map('#series'))}['My Series One']
         # Test custom column with rename to existing
         self.assertEqual(cache.rename_items('#series', {s:'My Series Two'})[0], {2})
         for c in (cache, self.init_cache(cl)):
@@ -584,7 +585,7 @@ class WritingTest(BaseTest):
 
         # Test renaming many-many items to multiple items
         cache = self.init_cache(self.cloned_library)
-        t = {v:k for k, v in cache.get_id_map('tags').iteritems()}['Tag One']
+        t = {v:k for k, v in iteritems(cache.get_id_map('tags'))}['Tag One']
         affected_books, id_map = cache.rename_items('tags', {t:'Something, Else, Entirely'})
         self.assertEqual({1, 2}, affected_books)
         tmap = cache.get_id_map('tags')
@@ -599,7 +600,7 @@ class WritingTest(BaseTest):
         # Test with restriction
         cache = self.init_cache()
         cache.set_field('tags', {1:'a,b,c', 2:'x,y,z', 3:'a,x,z'})
-        tmap = {v:k for k, v in cache.get_id_map('tags').iteritems()}
+        tmap = {v:k for k, v in iteritems(cache.get_id_map('tags'))}
         self.assertEqual(cache.rename_items('tags', {tmap['a']:'r'}, restrict_to_book_ids=()), (set(), {}))
         self.assertEqual(cache.rename_items('tags', {tmap['a']:'r', tmap['b']:'q'}, restrict_to_book_ids=(1,))[0], {1})
         self.assertEqual(cache.rename_items('tags', {tmap['x']:'X'}, restrict_to_book_ids=(2,))[0], {2})
@@ -644,7 +645,7 @@ class WritingTest(BaseTest):
         uv = int(cache.backend.user_version)
         all_ids = cache.all_book_ids()
         cache.dump_and_restore()
-        self.assertEqual(cache.set_field('title', {1:'nt'}), set([1]), 'database connection broken')
+        self.assertEqual(cache.set_field('title', {1:'nt'}), {1}, 'database connection broken')
         cache = self.init_cache()
         self.assertEqual(cache.all_book_ids(), all_ids, 'dump and restore broke database')
         self.assertEqual(int(cache.backend.user_version), uv)
@@ -653,11 +654,11 @@ class WritingTest(BaseTest):
     def test_set_author_data(self):  # {{{
         cache = self.init_cache()
         adata = cache.author_data()
-        ldata = {aid:str(aid) for aid in adata}
+        ldata = {aid:unicode_type(aid) for aid in adata}
         self.assertEqual({1,2,3}, cache.set_link_for_authors(ldata))
         for c in (cache, self.init_cache()):
-            self.assertEqual(ldata, {aid:d['link'] for aid, d in c.author_data().iteritems()})
-        self.assertEqual({3}, cache.set_link_for_authors({aid:'xxx' if aid == max(adata) else str(aid) for aid in adata}),
+            self.assertEqual(ldata, {aid:d['link'] for aid, d in iteritems(c.author_data())})
+        self.assertEqual({3}, cache.set_link_for_authors({aid:'xxx' if aid == max(adata) else unicode_type(aid) for aid in adata}),
                          'Setting the author link to the same value as before, incorrectly marked some books as dirty')
         sdata = {aid:'%s, changed' % aid for aid in adata}
         self.assertEqual({1,2,3}, cache.set_sort_for_authors(sdata))
@@ -708,7 +709,7 @@ class WritingTest(BaseTest):
         conn.execute('INSERT INTO tags (name) VALUES ("t")')
         norm = conn.last_insert_rowid()
         conn.execute('DELETE FROM books_tags_link')
-        for book_id, vals in {1:(lid, uid), 2:(uid, mid), 3:(lid, norm)}.iteritems():
+        for book_id, vals in iteritems({1:(lid, uid), 2:(uid, mid), 3:(lid, norm)}):
             conn.executemany('INSERT INTO books_tags_link (book,tag) VALUES (?,?)',
                              tuple((book_id, x) for x in vals))
         cache.reload_from_db()
